/*
 Copyright (C) 2007 Christian Dywan <christian@twotoasts.de>

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#ifndef __UI_H__
#define __UI_H__ 1

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>

#include "xbel.h"

typedef struct _CConfig
{
    guint startup; // 0: blank, 1: homepage, 2: last open pages
    gchar* homepage;
    gchar* locationSearch;
    gboolean toolbarNavigation;
    gboolean toolbarBookmarks;
    //gboolean toolbarDownloads;
    gboolean toolbarStatus;
    guint toolbarStyle;
    gboolean toolbarSmall;
    gboolean panelShow;
    guint panelActive;
    gchar* panelPageholder;
    // TODO: What about this? Support it or fix tab shrinking and drop it?
    guint tabSize; // tab size in charcters
    gboolean tabClose;
    gboolean toolbarWebSearch;
    gboolean toolbarNewTab;
    gboolean toolbarClosedTabs;
    guint newPages; // where to open new pages
    gboolean openTabsInTheBackground;
    gboolean openPopupsInTabs;

    gboolean loadImagesAutomatically;
    gboolean shrinkImagesToFit;
    gboolean resizableTextAreas;
    gboolean enableJavaScript;
    gboolean enableJava;
    gboolean enablePlugins;

    gboolean rememberWinMetrics; // Restore last state upon startup?
    gint winLeft;
    gint winTop;
    gint winWidth;
    gint winHeight;
    guint winPanelPos;
    guint searchEngine; // last selected search engine

    GPtrArray* protocols_names;
    GData*     protocols_commands;
} CConfig;

enum
{
    CONFIG_STARTUP_BLANK,
    CONFIG_STARTUP_HOMEPAGE,
    CONFIG_STARTUP_SESSION
};

enum
{
    CONFIG_TOOLBAR_DEFAULT,
    CONFIG_TOOLBAR_ICONS,
    CONFIG_TOOLBAR_TEXT,
    CONFIG_TOOLBAR_BOTH,
    CONFIG_TOOLBAR_BOTH_HORIZ
};

enum
{
    CONFIG_NEWPAGES_TAB_NEW,
    CONFIG_NEWPAGES_WINDOW_NEW,
    CONFIG_NEWPAGES_TAB_CURRENT
};

typedef struct _CBrowser
{
    // shared widgets
    GtkWidget* window;
    GtkActionGroup* actiongroup;
    // menus
    GtkWidget* menubar;
    GtkWidget* menu_tabsclosed;
    GtkWidget* menu_bookmarks;
    GtkWidget* menu_window;
    GtkWidget* popup_webView;
    GtkWidget* popup_element;
    GtkWidget* popup_editable;
    GtkWidget* throbber;
    // navibar
    GtkWidget* navibar;
    GtkWidget* newTab;
    GtkWidget* location_icon;
    GtkWidget* location;
    GtkWidget* webSearch;
    GtkWidget* closedTabs;
    GtkWidget* bookmarkbar;
    // panels
    GtkWidget* panels;
    GtkWidget* panels_notebook;
    GtkWidget* panel_pageholder;
    GtkWidget* webViews;
    // findbox
    GtkWidget* findbox;
    GtkWidget* findbox_text;
    GtkToolItem* findbox_case;
    GtkWidget* statusbar;
    GtkWidget* icon_link_script;
    GtkWidget* icon_link_window;
    GtkWidget* progress;
    GtkWidget* icon_security;
    GtkWidget* icon_newsfeed;

    // view specific widgets
    GtkWidget* webView_menu;
    GtkWidget* webView_icon;
    GtkWidget* webView_name;
    GtkWidget* webView_close;
    GtkWidget* webView;

    // view specific values
    gboolean hasMenubar;
    gboolean hasToolbar;
    gboolean hasLocation;
    gboolean hasStatusbar;
    gboolean isResizable;
    gchar* elementUri; // the element the mouse is hovering on
    gint loadedPercent; // -1 means "not loading"
    gint loadedBytes;
    gint loadedBytesMax;
    //UNDEFINED favicon;
    guint security;
    gchar* statusMessage; // message from a webView
    XBELitem* sessionItem;
} CBrowser;

typedef struct _CSearchEngine
{
    gchar* shortName;
    gchar* description;
    gchar* url;
    gchar* inputEncoding;
    gchar* icon;
    gchar* keyword;
} CSearchEngine;

// -- globals

CConfig* config;
GList* searchEngines; // Items of type 'CSearchEngine'
GList* browsers; // Items of type 'CBrowser'
GtkAccelGroup* accel_group;
XBELitem* bookmarks;
XBELitem* session;

// Custom stock items

// We should distribute these
// Names should match with epiphany and/ or xdg spec
/* NOTE: Those uncommented were replaced with remotely related icons
         in order to reduce the amount of warnings :D */

#define STOCK_BOOKMARK           GTK_STOCK_FILE // "stock_bookmark" // "bookmark-web"
#define STOCK_BOOKMARKS          "bookmark-view"
#define STOCK_DOWNLOADS          "package" // "download"
#define STOCK_CONSOLE            "terminal" // "console" // MISSING
#define STOCK_EXTENSIONS         "extension" // MISSING
#define STOCK_FORM_FILL          "insert-text" // "form-fill" // MISSING
#define STOCK_HISTORY            "document-open-recent"
#define STOCK_HISTORY_           "history-view"
#define STOCK_LOCATION           "location-entry"
#define STOCK_NEWSFEED           "gtk-index" // "newsfeed" // MISSING
#define STOCK_PLUGINS            "plugin" // MISSING
#define STOCK_POPUPS_BLOCKED     "popup-hidden"
#define STOCK_SOURCE_VIEW        "stock_view-html-source" // MISSING
#define STOCK_TAB_CLOSE          "tab-close" // MISSING
#define STOCK_WINDOW_CLOSE       "window-close" // MISSING

// We can safely use standard icons
// Assuming that we have reliable fallback icons

#define STOCK_BOOKMARK_NEW       "bookmark-new"
#define STOCK_BOOKMARK_NEW_      "stock_add-bookmark"
#define STOCK_HOMEPAGE           GTK_STOCK_HOME
#define STOCK_IMAGE              "image-x-generic"
#define STOCK_IMAGE_             "gnome-mime-image"
#define STOCK_LOCK_OPEN          "stock_lock-open"
#define STOCK_LOCK_SECURE        "stock_lock"
#define STOCK_LOCK_BROKEN        "stock_lock-broken"
#define STOCK_NETWORK_OFFLINE    "connect_no"
#define STOCK_NETWORK_OFFLINE_   "network-offline"
#define STOCK_SCRIPT             "stock_script"
#define STOCK_SEND               "mail-send"
#define STOCK_SEND_              "stock_mail-send"
#define STOCK_TAB_NEW            "tab-new"
#define STOCK_TAB_NEW_           "stock_new-tab"
#define STOCK_THEME              "gnome-settings-theme"
#define STOCK_USER_TRASH         "user-trash"
#define STOCK_USER_TRASH_        "gnome-stock-trash"
#define STOCK_WINDOW_NEW         "window-new"
#define STOCK_WINDOW_NEW_        "stock_new-window"

// For backwards compatibility

#if !GTK_CHECK_VERSION(2, 10, 0)
#define GTK_STOCK_SELECT_ALL     "gtk-select-all"
#endif

#endif /* !__UI_H__ */
