/*
 Copyright (C) 2007 Christian Dywan <christian@twotoasts.de>

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#ifndef __XBEL_H__
#define __XBEL_H__ 1

#include <stdio.h>
#include <string.h>
#include <glib.h>
#include <libxml/parser.h>
#include <libxml/tree.h>

#define XBEL_ERROR g_quark_from_string("XBEL_ERROR")

typedef enum
{
    XBEL_ERROR_INVALID_URI,        /* Malformed uri */
    XBEL_ERROR_INVALID_VALUE,      /* Requested field not found */
    XBEL_ERROR_URI_NOT_FOUND,      /* Requested uri not found */
    XBEL_ERROR_READ,               /* Malformed document */
    XBEL_ERROR_UNKNOWN_ENCODING,   /* Parsed text was in an unknown encoding */
    XBEL_ERROR_WRITE,              /* Writing failed. */
} XBELError;

typedef enum
{
    XBEL_ITEM_FOLDER,
    XBEL_ITEM_BOOKMARK,
    XBEL_ITEM_SEPARATOR
} XBELitemType;

// Note: This structure is entirely private.
typedef struct
{
    XBELitemType type;
    struct XBELitem* parent;

    GList* items; // folder
    gboolean folded; // foolder
    gchar* title;    // !separator
    gchar* desc;     // folder and bookmark
    gchar* href;     // bookmark
    //time_t added;    // !separator
    //time_t modfied;    // bookmark
    //time_t visited;    // bookmark
} XBELitem;

XBELitem* xbel_bookmark_new(void);

XBELitem* xbel_separator_new(void);

XBELitem* xbel_folder_new(void);

void xbel_item_free(XBELitem*);

XBELitem* xbel_item_copy(XBELitem*);

void xbel_folder_append_item(XBELitem*, XBELitem*);

XBELitem* xbel_folder_remove_item(XBELitem*, XBELitem*);

guint xbel_folder_get_n_items(XBELitem*);

XBELitem* xbel_folder_get_nth_item(XBELitem*, guint);

gboolean xbel_folder_get_folded(XBELitem*);

XBELitemType xbel_item_get_type(XBELitem*);

XBELitem* xbel_item_get_parent(XBELitem*);

G_CONST_RETURN gchar* xbel_item_get_title(XBELitem*);

G_CONST_RETURN gchar* xbel_item_get_desc(XBELitem*);

G_CONST_RETURN gchar* xbel_bookmark_get_href(XBELitem*);

//time_t xbel_bookmark_get_added(XBELitem*);

//time_t xbel_bookmark_get_modified(XBELitem*);

//time_t xbel_bookmark_get_visited(XBELitem*);

gboolean xbel_item_is_bookmark(XBELitem*);

gboolean xbel_item_is_separator(XBELitem*);

gboolean xbel_item_is_folder(XBELitem*);

void xbel_folder_set_folded(XBELitem*, gboolean);

void xbel_item_set_title(XBELitem*, const gchar*);

void xbel_item_set_desc(XBELitem*, const gchar*);

void xbel_bookmark_set_href(XBELitem*, const gchar*);

gboolean xbel_folder_from_data(XBELitem*, const gchar*, GError**);

gboolean xbel_folder_from_file(XBELitem*, const gchar*, GError**);

gboolean xbel_folder_from_data_dirs(XBELitem*, const gchar*, gchar**, GError**);

gchar* xbel_folder_to_data(XBELitem*, gsize*, GError**);

gboolean xbel_folder_to_file(XBELitem*, const gchar*, GError**);

#endif /* !__XBEL_H__ */
